// Copyright Vladimir Prus 2002. Permission to copy, use,
// modify, sell and distribute this software is granted provided this
// copyright notice appears in all copies. This software is provided
// "as is" without express or implied warranty, and with no claim as
// to its suitability for any purpose.

#ifndef ERRORS_VP_2003_01_02
#define ERRORS_VP_2003_01_02

#include <string>
#include <stdexcept>
#include <vector>

namespace boost { namespace program_options {

    /** Base class for all errors in the library. */
    class error : public std::logic_error {
    public:
        error(const std::string& what) : std::logic_error(what) {}
    };

    class invalid_syntax : public error {
    public:
        invalid_syntax(const std::string& tokens, const std::string& msg)
        : error(std::string(msg).append(" in '").append(tokens).append("'")),
          tokens(tokens), msg(msg)                       
        {}

        // gcc says that throw specification on dtor is loosened
        // without this line
        ~invalid_syntax() throw() {}

        // TODO: copy ctor might throw
        std::string tokens, msg;
    };

    /** Class thrown when option name is not recognized. */
    class unknown_option : public error {
    public:
        unknown_option(const std::string& name)
        : error(std::string("unknown option ").append(name)) 
        {}
    };

    /** Class thrown when there's ambiguity between several possible options. */
    class ambiguous_option : public error {
    public:
        ambiguous_option(const std::string& name, 
                         const std::vector<std::string>& alternatives)
        : error(std::string("ambiguous option ").append(name)),
          alternatives(alternatives)
        {}

        ~ambiguous_option() throw() {}

        // TODO: copy ctor might throw
        std::vector<std::string> alternatives;
    };

    /** Class thrown when there are several option values, but
        user called a method which cannot return them all. */
    class multiple_values : public error {
    public:
        multiple_values(const std::string& what) : error(what) {}
    };

    /** Class thrown when there are several occurences of an
        option, but user called a method which cannot return 
        them all. */
    class multiple_occurences : public error {
    public:
        multiple_occurences(const std::string& what) : error(what) {}
    };
}}

#endif
